import groovy.json.JsonGenerator
import groovy.json.JsonGenerator.Converter

nextflow.enable.dsl=2

// comes from nf-test to store json files
params.nf_test_output  = ""

// include dependencies

include { BBMAP_BBSPLIT as BBMAP_BBSPLIT_INDEX } from '/workspace/modules/./modules/nf-core/bbmap/bbsplit/tests/../main.nf'


// include test process
include { BBMAP_BBSPLIT } from '/workspace/modules/./modules/nf-core/bbmap/bbsplit/tests/../main.nf'

// define custom rules for JSON that will be generated.
def jsonOutput =
    new JsonGenerator.Options()
        .excludeNulls()  // Do not include fields with value null..
        .addConverter(Path) { value -> value.toString() } // Custom converter for Path. Only filename
        .build()


workflow {

    // run dependencies
    
    {
        def input = []
        
                    input = [[:],[]]
                    index = []
                    fasta = file(params.test_data['sarscov2']['genome']['genome_fasta'], checkIfExists: true)
                    bbsplit_fasta_list = [
                        ['human'],
                        file('https://raw.githubusercontent.com/nf-core/test-datasets/rnaseq/reference/chr22_23800000-23980000.fa', checkIfExists: true)
                    ]
                    only_build_index = true

                    input[0] = input
                    input[1] = index
                    input[2] = fasta
                    input[3] = bbsplit_fasta_list
                    input[4] = only_build_index
                    
        BBMAP_BBSPLIT as BBMAP_BBSPLIT_INDEX(*input)
    }
    

    // process mapping
    def input = []
    
                    input = [
                        [ id:'test', single_end:true ], // meta map
                        file(params.test_data['sarscov2']['illumina']['test_1_fastq_gz'], checkIfExists: true)
                    ]
                    index = BBMAP_BBSPLIT_INDEX.out.index
                    fasta = file(params.test_data['sarscov2']['genome']['genome_fasta'], checkIfExists: true)
                    bbsplit_fasta_list = [
                        ['human'],
                        file('https://raw.githubusercontent.com/nf-core/test-datasets/rnaseq/reference/chr22_23800000-23980000.fa', checkIfExists: true)
                    ]
                    only_build_index = true

                    input[0] = input
                    input[1] = index
                    input[2] = fasta
                    input[3] = bbsplit_fasta_list
                    input[4] = only_build_index
                    
    //----

    //run process
    BBMAP_BBSPLIT(*input)

    if (BBMAP_BBSPLIT.output){

        // consumes all named output channels and stores items in a json file
        for (def name in BBMAP_BBSPLIT.out.getNames()) {
            serializeChannel(name, BBMAP_BBSPLIT.out.getProperty(name), jsonOutput)
        }	  
      
        // consumes all unnamed output channels and stores items in a json file
        def array = BBMAP_BBSPLIT.out as Object[]
        for (def i = 0; i < array.length ; i++) {
            serializeChannel(i, array[i], jsonOutput)
        }    	

    }
  
}

def serializeChannel(name, channel, jsonOutput) {
    def _name = name
    def list = [ ]
    channel.subscribe(
        onNext: {
            list.add(it)
        },
        onComplete: {
              def map = new HashMap()
              map[_name] = list
              def filename = "${params.nf_test_output}/output_${_name}.json"
              new File(filename).text = jsonOutput.toJson(map)		  		
        } 
    )
}


workflow.onComplete {

    def result = [
        success: workflow.success,
        exitStatus: workflow.exitStatus,
        errorMessage: workflow.errorMessage,
        errorReport: workflow.errorReport
    ]
    new File("${params.nf_test_output}/workflow.json").text = jsonOutput.toJson(result)
    
}
